@@ -12,7 +12,7 @@ use warnings;
 # It is provided simply to keep CPAN happy:
 #   cpan -i IO::Async
 
-our $VERSION = '0.46';
+our $VERSION = '0.47';
 
 =head1 NAME
 
@@ -54,7 +54,7 @@ C<IO::Async> - Asynchronous event-driven programming
     on_connect_error => sub { die "Cannot connect - $_[0] failed $_[-1]\n"; },
  );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head1 DESCRIPTION
 
@@ -147,10 +147,8 @@ L<IO::Async::Loop::Poll> uses an L<IO::Poll> object for this test.
 
 L<IO::Async::Loop::Select> uses the C<select(2)> syscall.
 
-Other subclasses of loop may appear on CPAN under their own dists; such
-as L<IO::Async::Loop::Glib> which acts as a proxy for the C<Glib::MainLoop> of
-a L<Glib>-based program, or L<IO::Async::Loop::Ppoll> which uses the
-L<IO::Ppoll> object to handle signals safely on Linux.
+Other subclasses of loop may appear on CPAN under their own dists; see the
+L</SEE ALSO> section below for more detail.
 
 As well as these general-purpose classes, the L<IO::Async::Loop> constructor
 also supports looking for OS-specific subclasses, in case a more efficient
@@ -267,6 +265,70 @@ Bugs may be reported via RT at
 Support by IRC may also be found on F<irc.perl.org> in the F<#io-async>
 channel.
 
+=cut
+
+=head1 SEE ALSO
+
+As well as the two loops supplied in this distribution, many more exist on
+CPAN. At the time of writing this includes:
+
+=over 4
+
+=item *
+
+L<IO::Async::Loop::AnyEvent> - use IO::Async with AnyEvent
+
+=item *
+
+L<IO::Async::Loop::Epoll> - use IO::Async with epoll on Linux
+
+=item *
+
+L<IO::Async::Loop::EV> - use IO::Async with EV
+
+=item *
+
+L<IO::Async::Loop::Glib> - use IO::Async with Glib or GTK
+
+=item *
+
+L<IO::Async::Loop::KQueue> - use IO::Async with kqueue
+
+=item *
+
+L<IO::Async::Loop::Mojo> - use IO::Async with Mojolicious
+
+=item *
+
+L<IO::Async::Loop::POE> - use IO::Async with POE
+
+=item *
+
+L<IO::Async::Loop::Ppoll> - use IO::Async with ppoll(2)
+
+=back
+
+Additionally, some other event loops or modules also support being run on top
+of C<IO::Async>:
+
+=over 4
+
+=item *
+
+L<AnyEvent::Impl::IOAsync> - AnyEvent adapter for IO::Async
+
+=item * 
+
+L<Gungho::Engine::IO::Async> - IO::Async Engine
+
+=item *
+
+L<POE::Loop::IO_Async> - IO::Async event loop support for POE
+
+=back
+
+=cut
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
