@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.46';
+our $VERSION = '0.47';
 
 use Carp;
 
@@ -33,13 +33,13 @@ C<IO::Async::Timer::Absolute> - event callback at a fixed future time
 
     on_expire => sub {
        print "It's midnight\n";
-       $loop->loop_stop;
+       $loop->stop;
     },
  );
 
  $loop->add( $timer );
 
- $loop->loop_forever;
+ $loop->run;
 
 =head1 DESCRIPTION
 
